@@ -6,6 +6,39 @@
 //!
 //! Based on paper [Last Order Logic](https://github.com/advancedresearch/path_semantics/blob/master/papers-wip2/last-order-logic.pdf).
 //!
+//! ```text
+//! === Last Order Logic 0.2 ===
+//! Type `help` for more information.
+//! > a := 1 ~= 0
+//! a := 1 ~= 0
+//! LOL: Added `a` to definitions
+//! > a ~ 0
+//! a ~ 0
+//! > ty
+//! 1
+//! ```
+//!
+//! To run LOL from your Terminal, type:
+//!
+//! ```text
+//! cargo install --example lolz last_order_logic
+//! ```
+//!
+//! Then, to run:
+//!
+//! ```text
+//! lolz
+//! ```
+//!
+//! ### How to learn LOL
+//!
+//! To learn how to use LOL, type "help" in LOLZ.
+//! This command lists all topics, e.g. "help path".
+//!
+//! ### Examples
+//!
+//! - [Geometry Synthesis](https://github.com/advancedresearch/last_order_logic/blob/main/source/geom.lol.md)
+//!
 //! ### Motivation
 //!
 //! In [First Order Logic](https://en.wikipedia.org/wiki/First-order_logic),
@@ -29,7 +62,7 @@
 //! Last Order Logic fixes this problem by having quantified expressions evaluate to themselves,
 //! while the truth value is encoded in the type.
 //!
-//! `∀ x { ... } : un(1)` - It is easy to see this is `true`.
+//! `∀ x : I { ... } : un(1)` - It is easy to see this is `true`.
 //!
 //! Types are used to communicate intentions of programs.
 //! Last Order Logic uses this feature to increase readability.
@@ -44,6 +77,19 @@
 //! The distinction between uniform and non-uniform sense of truth comes from the theory of
 //! [Avatar Extensions](https://advancedresearch.github.io/avatar-extensions/summary.html).
 //! Only non-uniform truth has a meaningful example that shows its truth value.
+//!
+//! ### File formats
+//!
+//! - Regular text format (`.lol.txt`)
+//! - Markdown text format (`.lol.md`)
+//!
+//! The Markdown format is designed for readability:
+//!
+//! - Must start with `#` (markdown title)
+//! - A codeblock must use 3 backticks and `lol`
+//!
+//! Special commands in the LOLZ environment are not supported.
+//! To refer to LOLZ usage, use 3 backticks and `text`.
 
 use std::sync::Arc;
 use Expr::*;
@@ -54,6 +100,9 @@ pub const T: Expr = _1;
 /// Alternative for `false/0`.
 pub const F: Expr = _0;
 
+pub mod parsing;
+pub mod runtime;
+
 /// Stores an expression.
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub enum Expr {
@@ -67,6 +116,8 @@ pub enum Expr {
     Var(Arc<String>),
     /// Type judgement.
     Ty(Box<Expr>, Box<Expr>),
+    /// Type evaluation.
+    Type(Box<Expr>),
     /// Path.
     Pa(Box<Expr>, Box<Expr>),
     /// Lambda expression.
@@ -107,6 +158,7 @@ impl fmt::Display for Expr {
             _0 => write!(w, "0")?,
             _1 => write!(w, "1")?,
             I => write!(w, "I")?,
+            Type(a) => write!(w, "type({})", a)?,
             Un(a) => write!(w, "un({})", a)?,
             Nu(a) => write!(w, "nu({})", a)?,
             Var(name) => write!(w, "{}", name)?,
@@ -149,7 +201,11 @@ impl fmt::Display for Expr {
                 }
             }
             App(a, b) => {
-                write!(w, "{}({})", a, b)?;
+                if a.needs_parens() {
+                    write!(w, "({})({})", a, b)?
+                } else {
+                    write!(w, "{}({})", a, b)?
+                };
             }
             Lift(a) => {
                 write!(w, "lift({})", a)?;
@@ -182,7 +238,6 @@ impl fmt::Display for Expr {
                 write!(w, " ⊻ ")?;
                 if b.needs_parens() {write!(w, "({})", b)?} else {write!(w, "{}", b)?};
             }
-            // _ => unimplemented!("{:?}", self),
         }
         Ok(())
     }
@@ -193,7 +248,7 @@ impl Expr {
     pub fn needs_parens(&self) -> bool {
         match self {
             _1 | _0 | I | Var(_) | Un(_) | Nu(_) |
-            All(_) | Any(_) | Tup(_) | Not(_) | Lift(_) => false,
+            All(_) | Any(_) | Tup(_) | Not(_) | Lift(_) | App(_, _) => false,
             _ => true,
         }
     }
@@ -202,24 +257,13 @@ impl Expr {
     pub fn is_false(&self) -> bool {
         match self {
             _0 => true,
-            _1 => false,
-            I => false,
-            Var(_) => false,
             Un(a) | Nu(a) => a.is_false(),
-            _ => unimplemented!("{}", self),
+            _ => false,
         }
     }
 
     /// Returns `true` if the expression is truish expression, `false` otherwise.
-    pub fn is_true(&self) -> bool {
-        match self {
-            _0 => false,
-            _1 => true,
-            Var(_) => true,
-            Un(a) | Nu(a) => a.is_true(),
-            _ => unimplemented!("{:?}", self),
-        }
-    }
+    pub fn is_true(&self) -> bool {!self.is_false()}
 
     /// Gets the members of some type.
     pub fn members(&self) -> Vec<Expr> {
@@ -243,8 +287,9 @@ impl Expr {
     /// Gets the type of an expression.
     pub fn ty(&self) -> Option<Expr> {
         match self {
-            _0 | _1 => Some(I),
-            Var(_) => None,
+            _0 | _1 | Ty(_, _) => Some(I),
+            I | Var(_) | Lam(_, _) => None,
+            Type(a) => a.eval().ty()?.ty(),
             Ind(p, i) => {
                 if let Pa(a, b) = &**p {
                     match &**i {
@@ -263,6 +308,17 @@ impl Expr {
                     None
                 }
             }
+            Pa(a, b) => Some(pa(a.ty()?, b.ty()?)),
+            App(a, b) => a.app(b)?.ty(),
+            Un(a) => Some(un(a.ty()?)),
+            Nu(a) => Some(nu(a.ty()?)),
+            Tup(vs) => {
+                let mut res = vec![];
+                for item in vs {
+                    res.push(item.ty()?);
+                }
+                Some(Tup(res))
+            }
             All(lam) => {
                 if let Lam(arg, _) = &**lam {
                     if let Ty(_, ty) = &**arg {
@@ -317,11 +373,12 @@ impl Expr {
                 }
             }
             Lift(a) => Some(a.eval()),
-            Not(a) => {
-                let a_ty = a.ty()?;
-                Some(not(a_ty).eval())
-            }
-            _ => unimplemented!("{}", self),
+            Not(a) => Some(not(a.ty()?).eval()),
+            And(a, b) => Some(and(a.ty()?, b.ty()?).eval()),
+            Or(a, b) => Some(or(a.ty()?, b.ty()?).eval()),
+            Imply(a, b) => Some(imply(a.ty()?, b.ty()?).eval()),
+            Eq(a, b) => Some(eq(a.ty()?, b.ty()?).eval()),
+            Xor(a, b) => Some(xor(a.ty()?, b.ty()?).eval()),
         }
     }
 
@@ -331,6 +388,10 @@ impl Expr {
         else {
             match self {
                 _0 | _1 | I | Var(_) => (self.clone(), 0),
+                Type(a) => {
+                    let (a2, n) = a.substitute(arg, v);
+                    (typ(a2), n)
+                }
                 Un(a) => {
                     let (a2, n) = a.substitute(arg, v);
                     (un(a2), n)
@@ -355,11 +416,12 @@ impl Expr {
                     (pa(a2, b2), n + m)
                 }
                 Lam(arg2, body) => {
-                    if let Ty(a, _) = &**arg2 {
-                        if **a == *arg {(self.clone(), 0)}
+                    if let Ty(a, a_ty) = &**arg2 {
+                        let (a_ty2, m) = a_ty.substitute(arg, v);
+                        if **a == *arg {(lam(ty((**a).clone(), a_ty2), (**body).clone()), m)}
                         else {
                             let (body2, n) = body.substitute(arg, v);
-                            (lam((**arg2).clone(), body2), n)
+                            (lam(ty((**a).clone(), a_ty2), body2), m + n)
                         }
                     } else {
                         (self.clone(), 0)
@@ -385,7 +447,7 @@ impl Expr {
                 }
                 Lift(a) => {
                     let (a2, n) = a.substitute(arg, v);
-                    (lift(a2), n + 1)
+                    (lift(a2), n)
                 }
                 App(a, b) => {
                     let (a2, n) = a.substitute(arg, v);
@@ -421,7 +483,6 @@ impl Expr {
                     let (b2, m) = b.substitute(arg, v);
                     (xor(a2, b2), n + m)
                 }
-                // _ => unimplemented!("{}", self)
             }
         }
     }
@@ -430,10 +491,17 @@ impl Expr {
     pub fn app(&self, v: &Expr) -> Option<Expr> {
         match self {
             Lam(arg, body) => {
-                if let Ty(a, _) = &**arg {
-                    let (body2, n) = body.substitute(a, v);
-                    if n == 0 {None}
-                    else {Some(body2.eval())}
+                if let Ty(a, arg_ty) = &**arg {
+                    if let Pa(v0, v1) = v {
+                        if let Some(v_ty) = v.ty() {
+                            if &v_ty != &**arg_ty {
+                                return Some(pa(app(self.clone(), (**v0).clone()),
+                                               app(self.clone(), (**v1).clone())).eval());
+                            }
+                        }
+                    };
+                    let (body2, _) = body.substitute(a, v);
+                    Some(body2.eval())
                 } else {
                     None
                 }
@@ -446,6 +514,7 @@ impl Expr {
     pub fn eval(&self) -> Expr {
         match self {
             _0 | _1 | I | Var(_) => self.clone(),
+            Type(a) => if let Some(ty) = a.ty() {ty} else {self.clone()},
             Un(a) => un(a.eval()),
             Nu(a) => nu(a.eval()),
             Pa(a, b) => pa(a.eval(), b.eval()),
@@ -453,6 +522,10 @@ impl Expr {
             Ind(p, i) => ind(p.eval(), i.eval()),
             Any(lam) => Any(Box::new(lam.eval())),
             All(lam) => All(Box::new(lam.eval())),
+            App(a, b) => {
+                let b2 = b.eval();
+                if let Some(res) = a.app(&b2) {res} else {self.clone()}
+            }
             Lift(a) => lift(a.eval()),
             Tup(tup) => Tup(tup.iter().map(|n| n.eval()).collect()),
             Ty(a, b) => {
@@ -461,13 +534,14 @@ impl Expr {
                 if let Some(a_ty) = a2.ty() {
                     if a_ty == b2 {T} else {F}
                 } else {
-                    F
+                    self.clone()
                 }
             }
             Not(a) => {
                 match &**a {
                     _0 => _1,
                     _1 => _0,
+                    I => I,
                     Un(a) => un(not((**a).clone()).eval()).eval(),
                     Nu(a) => nu(not((**a).clone()).eval()).eval(),
                     Pa(a, b) => pa(not((**a).clone()).eval(), not((**b).clone()).eval()),
@@ -485,6 +559,7 @@ impl Expr {
                 match (&**a, &**b) {
                     (_1, _1) => _1,
                     (_0, _) | (_, _0) => _0,
+                    (I, I) => I,
                     (Pa(a0, a1), Pa(b0, b1)) => pa(
                         and((**a0).clone(), (**b0).clone()).eval(),
                         and((**a1).clone(), (**b1).clone()).eval(),
@@ -501,6 +576,7 @@ impl Expr {
                 match (&**a, &**b) {
                     (_1, _) | (_, _1) => _1,
                     (_0, _0) => _0,
+                    (I, I) => I,
                     (Pa(a0, a1), Pa(b0, b1)) => pa(
                         or((**a0).clone(), (**b0).clone()).eval(),
                         or((**a1).clone(), (**b1).clone()).eval(),
@@ -517,6 +593,7 @@ impl Expr {
                 match (&**a, &**b) {
                     (_0, _) | (_1, _1) => _1,
                     (_1, _0) => _0,
+                    (I, I) => I,
                     (Pa(a0, a1), Pa(b0, b1)) => pa(
                         imply((**a0).clone(), (**b0).clone()).eval(),
                         imply((**a1).clone(), (**b1).clone()).eval(),
@@ -533,6 +610,7 @@ impl Expr {
                 match (&**a, &**b) {
                     (_0, _0) | (_1, _1) => _1,
                     (_1, _0) | (_0, _1) => _0,
+                    (I, I) => I,
                     (Pa(a0, a1), Pa(b0, b1)) => pa(
                         eq((**a0).clone(), (**b0).clone()).eval(),
                         eq((**a1).clone(), (**b1).clone()).eval(),
@@ -549,6 +627,7 @@ impl Expr {
                 match (&**a, &**b) {
                     (_0, _0) | (_1, _1) => _0,
                     (_1, _0) | (_0, _1) => _1,
+                    (I, I) => I,
                     (Pa(a0, a1), Pa(b0, b1)) => pa(
                         xor((**a0).clone(), (**b0).clone()).eval(),
                         xor((**a1).clone(), (**b1).clone()).eval(),
@@ -561,7 +640,6 @@ impl Expr {
                     }
                 }
             }
-            _ => unimplemented!("{}", self),
         }
     }
 }
@@ -570,6 +648,10 @@ impl From<&str> for Expr {
     fn from(v: &str) -> Expr {Var(Arc::new(v.into()))}
 }
 
+impl From<Arc<String>> for Expr {
+    fn from(v: Arc<String>) -> Expr {Var(v)}
+}
+
 /// `f(a)` - Lambda application.
 pub fn app<T: Into<Expr>, U: Into<Expr>>(a: T, b: U) -> Expr {
     App(Box::new(a.into()), Box::new(b.into()))
@@ -590,6 +672,11 @@ pub fn ty<T: Into<Expr>, U: Into<Expr>>(a: T, b: U) -> Expr {
     Ty(Box::new(a.into()), Box::new(b.into()))
 }
 
+/// `type(a)` - Type evaluation.
+pub fn typ<T: Into<Expr>>(a: T) -> Expr {
+    Type(Box::new(a.into()))
+}
+
 /// `p ~ i` - Path index.
 pub fn ind<T: Into<Expr>, U: Into<Expr>>(a: T, b: U) -> Expr {
     Ind(Box::new(a.into()), Box::new(b.into()))
@@ -676,6 +763,12 @@ mod tests {
 
         let b = ind(pa("a", "b"), _1).ty();
         assert_eq!(b, Some("b".into()));
+
+        let e = pa(_0, _0);
+        assert_eq!(e.ty(), Some(pa(I, I)));
+
+        let e = pa(pa(_0, _0), _0);
+        assert_eq!(e.ty(), Some(pa(pa(I, I), I)));
     }
 
     #[test]
@@ -719,6 +812,9 @@ mod tests {
 
         let a = not(T).eval();
         assert_eq!(a, F);
+
+        let a = not(I).eval();
+        assert_eq!(a, I);
     }
 
     #[test]
@@ -756,10 +852,25 @@ mod tests {
     #[test]
     fn test_fail() {
         let a = all(ty("i", I), T);
-        assert_eq!(a.ty(), None);
+        assert_eq!(a.ty(), Some(un(I)));
 
         let a = all2(ty("i", I), ty("j", I), ind(pa(T, T), "i"));
-        assert_eq!(a.ty(), None);
+        assert_eq!(a.ty(), Some(un(un(T))));
+    }
+
+    #[test]
+    fn test_ty_ty() {
+        let e = ty(_0, I);
+        assert_eq!(e.ty(), Some(I));
+
+        assert_eq!(I.ty(), None);
+
+        assert_eq!(nu(_0).ty(), Some(nu(I)));
+        assert_eq!(un(_0).ty(), Some(un(I)));
+
+        assert_eq!(tup2(_0, _1).ty(), Some(tup2(I, I)));
+
+        assert_eq!(typ(ind(pa(_0, _1), _1)).ty(), Some(I));
     }
 
     #[test]
@@ -807,6 +918,20 @@ mod tests {
         assert_eq!(a.ty(), Some(un(T)));
     }
 
+    #[test]
+    fn test_app() {
+        let e = app(lam(ty("a", I), "a"), _0);
+        assert_eq!(e.eval(), _0);
+        assert_eq!(e.ty(), Some(I));
+
+        let e = app(lam(ty("a", pa(I, I)), ind("a", _0)), pa(_1, _0));
+        assert_eq!(e.eval(), ind(pa(_1, _0), _0));
+        assert_eq!(e.ty(), Some(_1));
+
+        let e = app(lam(ty("p", pa(I, I)), ty(all(ty("i", I), ind("p", "i")), un(_1))), pa(_1, _1));
+        assert_eq!(e.eval(), _1);
+    }
+
     #[test]
     fn test_eval_ty() {
         let e = ty(ind(pa(T, F), _0), T);
@@ -900,6 +1025,9 @@ mod tests {
         let a = format!("{}", ty(all2(ty("i", I), ty("j", I),
             ind(pa(pa(T, T), pa(T, T)), tup2("i", "j"))), un(un(T))));
         assert_eq!(a, "∀ i : I { ∀ j : I { ((1 ~= 1) ~= (1 ~= 1)) ~ (i, j) } } : un(un(1))");
+
+        let a = format!("{}", app(lam(ty("a", I), "a"), _0));
+        assert_eq!(a, "(\\(a : I) = a)(0)");
     }
 
     #[test]
@@ -909,6 +1037,9 @@ mod tests {
 
         let a = all(ty("i", I), ind(pa("a", "a"), "i"));
         assert_eq!(a.ty(), Some(un("a")));
+
+        let a = all(ty("p", pa(I, I)), lift(and("p", "p")));
+        assert_eq!(a.ty(), None);
     }
 
     #[test]
@@ -916,4 +1047,12 @@ mod tests {
         let a = any(ty("i", I), ind(pa("a", "b"), "i"));
         assert_eq!(a.ty(), None);
     }
+
+    #[test]
+    fn test_eval() {
+        let e = parsing::parse_str(r#"(\(p : ((I ~= I) ~= (I ~= I))) =
+        (\(p : (I ~= I)) = ∀ i : I { p ~ i } : un(1))((\(p : (I ~= I)) =
+        ∀ i : I { p ~ i } : un(1))(p)))((1 ~= 1) ~= (1 ~= 1))"#).unwrap();
+        assert_eq!(e.eval(), _1);
+    }
 }
